/*******************************************************************************
 * The converter service is used to take in different weather measurements
 * and conver them from one unit of measure to another.
 *
 * TODO use JQuery number formatting extension for i18n:
 *
 * @see http://plugins.jquery.com/project/numberformatter
 * @see /content/logging.js
 * @see /content/observers.js
 * @see /content/preferences.js
 * @see /content/external/class.js
 * @see /content/external/jquery-1.3.2.js
 * @version 1.0
 ******************************************************************************/const SYSTEM_AMERICAN = {
    temperature: [ "f" ],
    speed: [ "mph" ],
    time: [ "h12" ],
    pressure: [ "inhg" ],
    distance: {
        shrt: [ "inches" ],
        lng: [ "mi" ]
    }
}, SYSTEM_METRIC = {
    temperature: [ "c" ],
    speed: [ "kph" ],
    time: [ "h12" ],
    pressure: [ "mmhg" ],
    distance: {
        shrt: [ "cm" ],
        lng: [ "km" ]
    }
}, CONVERTERS = {
    temperature: {
        _count: 4,
        c: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.temperature.c"), {
                temperature: Math.round((Number(value) - 32) * (5 / 9))
            });
        },
        f: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.temperature.f"), {
                temperature: Math.round(Number(value))
            });
        },
        r: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.temperature.r"), {
                temperature: Math.round(Number(value) + 459.67)
            });
        },
        k: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.temperature.k"), {
                temperature: Math.round((Number(value) - 32) * (5 / 9) + 273.15)
            });
        }
    },
    speed: {
        _count: 4,
        mph: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.speed.mph"), {
                speed: Math.round(Number(value))
            });
        },
        kph: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.speed.kph"), {
                speed: Math.round(Number(value) * 1.609)
            });
        },
        ms: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.speed.ms"), {
                speed: Math.round(Number(value) * .447)
            });
        },
        bft: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.speed.bft"), {
                speed: Math.round(Math.pow(value * .5348, 2 / 3))
            });
        },
        knots: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.speed.knots"), {
                speed: Math.round(Number(value) * .868976242)
            });
        }
    },
    time: {
        _count: 2,
        h12: function(value, i18n, templator) {
            var nums = value.split(":"), h = Number(nums[0]), m = nums[1];
            m.length == 1 && (m = "0" + m);
            var p = h < 12 || h == 24 ? "am" : "pm";
            return h > 12 && (h -= 12), h == 0 && (h = 12), isNaN(h) ? "--:--" : templator.render(i18n.data("converters.time.h12." + p), {
                hours: h,
                minutes: m
            });
        },
        h24: function(value, i18n, templator) {
            var nums = value.split(":"), h = Number(nums[0]);
            h < 10 && (h = "0" + ("" + h));
            var m = nums[1];
            return m.length == 1 && (m = "0" + m), templator.render(i18n.data("converters.time.h24"), {
                hours: h,
                minutes: m
            });
        }
    },
    pressure: {
        _count: 5,
        inhg: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.pressure.inhg"), {
                pressure: Number(value).toFixed(2)
            });
        },
        psi: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.pressure.psi"), {
                pressure: (Number(value) * .491098).toFixed(2)
            });
        },
        mb: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.pressure.mb"), {
                pressure: Math.round(Number(value) * 33.86)
            });
        },
        hpa: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.pressure.hpa"), {
                pressure: Math.round(Number(value) * 33.86)
            });
        },
        mmhg: function(value, i18n, templator) {
            return templator.render(i18n.data("converters.pressure.mmhg"), {
                pressure: Math.round(Number(value) * 25.39709)
            });
        }
    },
    distance: {
        lng: {
            _count: 3,
            mi: function(value, i18n, templator) {
                return templator.render(i18n.data("converters.distance.lng.mi"), {
                    distance: Math.round(Number(value))
                });
            },
            km: function(value, i18n, templator) {
                return templator.render(i18n.data("converters.distance.lng.km"), {
                    distance: Math.round(Number(value) * 1.6093)
                });
            },
            m: function(value, i18n, templator) {
                return templator.render(i18n.data("converters.distance.lng.m"), {
                    distance: Math.round(Number(value) * 1609.344)
                });
            },
            nm: function(value, i18n, templator) {
                return templator.render(i18n.data("converters.distance.lng.nm"), {
                    distance: Math.round(Number(value) * .868976242)
                });
            }
        },
        shrt: {
            _count: 3,
            inches: function(value, i18n, templator) {
                return templator.render(i18n.data("converters.distance.shrt.inches"), {
                    distance: Number(value).toFixed(2)
                });
            },
            cm: function(value, i18n, templator) {
                return templator.render(i18n.data("converters.distance.shrt.cm"), {
                    distance: (Number(value) * 2.54).toFixed(2)
                });
            },
            mm: function(value, i18n, templator) {
                return templator.render(i18n.data("converters.distance.shrt.mm"), {
                    distance: (Number(value) * 25.4).toFixed(2)
                });
            }
        }
    },
    percent: function(value, i18n, templator) {
        return value = $.trim(new String(value)), value[value.length - 1] == "%" && (value = value.substring(value, value.length - 1)), templator.render(i18n.data("converters.percent"), {
            percent: Math.round(Number(value))
        });
    },
    coordinate: function(value, i18n, templator) {
        return templator.render(i18n.data("converters.coordinate"), {
            coordinate: Number(value).toFixed(2)
        });
    }
};

var Converter = Class.extend({
    _systemkey: "unitsystem",
    _key: "units",
    _logger: null,
    _preferences: null,
    _i18n: null,
    _templator: null,
    init: function(logging, preferences, i18n, templator, units) {
        this._logger = logging.getLogger("converter"), this._preferences = preferences, this._i18n = i18n, this._templator = templator, units && this.units(units), this._logger.debug("initialized");
    },
    units: function(units) {
        var system = this._preferences.preference(this._systemkey), current = null;
        switch (system) {
          case "american":
            current = SYSTEM_AMERICAN;
            break;
          case "metric":
            current = SYSTEM_METRIC;
            break;
          case "custom":
          default:
            current = this._preferences.preference(this._key);
        }
        return units && this._validate(units) && (this._logger.debug("saving new unit preferences " + JSON.stringify(units)), this._preferences.preference(this._key, units), current = units), current;
    },
    temperature: function(value, join) {
        return this._convert("temperature", value, join);
    },
    wind: function(value) {
        return this._convert("speed", value, " / ");
    },
    distance: function(value, isLong) {
        return this._convert("distance", value, " / ", isLong ? "lng" : "shrt");
    },
    pressure: function(value) {
        return this._convert("pressure", value, " / ");
    },
    time: function(value) {
        return this._convert("time", value, " / ");
    },
    percent: function(value) {
        return CONVERTERS.percent(value, this._i18n, this._templator);
    },
    coordinate: function(value) {
        return CONVERTERS.coordinate(value, this._i18n, this._templator);
    },
    link: function(value) {
        return value;
    },
    description: function(value) {
        return value.charAt(0).toUpperCase() + value.slice(1);
    },
    _validate: function(units) {
        try {
            this._validate_array(units, [ "temperature" ]), this._validate_array(units, [ "speed" ]), this._validate_array(units, [ "time" ]), this._validate_array(units, [ "pressure" ]), this._validate_array(units, [ "distance", "lng" ]), this._validate_array(units, [ "distance", "shrt" ]);
        } catch (e) {
            return this._logger.exception("units failed validation " + JSON.stringify(units), e), !1;
        }
        for (var key in units) if (!(key in CONVERTERS)) return this._logger.error("units failed validation.  unit type not valid " + key + " " + JSON.stringify(units)), !1;
        return !0;
    },
    _validate_array: function(units, path) {
        var cv = CONVERTERS, attr, x;
        for (x = 0; x < path.length; x++) {
            attr = path[x], cv = cv[attr];
            if (attr in units) units = units[attr]; else throw this._logger.error("units failed validation, " + attr + " not present"), "missing units " + attr;
        }
        if (!$.isArray(units)) throw this._logger.error("units " + attr + " not an array"), "invalid unit format, " + attr + " not an array";
        if (units.length > cv._count) throw this._logger.error("unit preference has more units than we support " + JSON.stringify(units)), "too many units";
        if (units.length == 0) throw "no unit specified for " + path.toString();
        for (x = 0; x < units.length; x++) if (!(units[x] in cv)) throw this._logger.error("invalid unit for " + path.toString() + " " + units[x]), "invalid unit " + path.toString() + " " + units[x];
    },
    _convert: function(type, value, join, subtype) {
        var units = this.units();
        units = subtype ? units[type][subtype] : units[type];
        var converter = subtype ? CONVERTERS[type][subtype] : CONVERTERS[type], self = this, values = $.map(units, function(pref) {
            return converter[pref](value, self._i18n, self._templator);
        });
        return join ? values.join(join) : values;
    }
});