/*******************************************************************************
 * Objects used by the front-end to take the backing data and render a feature.
 * A feature controls the display of a specific block of backing data.
 *  Each feature to display should subclass the feature object.
 *
 *
 * @see /content/logging.js
 * @see /content/preferences.js
 * @see /content/observers.js
 * @see /content/templator.js
 * @see /content/weather-data-provider.js
 * @see /content/external/class.js
 * @see /content/external/jquery-1.3.2.js
 * @version 1.0
 ******************************************************************************/const XUL_NS_URI = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var Feature = Class.extend({
    _id: null,
    _logger: null,
    _preferences: null,
    _i18n: null,
    _themes: null,
    disabled: !1,
    init: function(logging, preferences, i18n, themes) {
        this._logger = logging.getLogger(this._id), this._preferences = preferences, this._i18n = i18n, this._themes = themes, this._logger.debug(this._id + " initialized");
    },
    preferences: function() {
        return this._preferences.preferences();
    },
    data: function() {
        return {};
    },
    message: function(templator, template) {
        return this._logger.debug("rendering " + this._id + " message"), "";
    },
    menu: function(templator, template) {
        return this._logger.debug("rendering " + this._id + " menu"), "";
    },
    details: function(template) {
        return this._logger.debug("rendering " + this._id + " details"), "";
    },
    toolbar: function(template) {
        this._logger.debug("rendering " + this._id + " toolbar");
        return;
    },
    _create_panel: function(templator, template, data) {
        this._logger.debug("Creating panel for " + this._id);
        var panel = templator._document.createElement(template);
        return "image" in data && panel.setAttribute("image", data.image), "defaultimage" in data && panel.setAttribute("imageerr", 'this.onerror = null; this.src = "' + data.defaultimage + '"'), "label" in data && panel.setAttribute("label", data.label), "link" in data && panel.setAttribute("link", data.link.replace("&amp;", "&")), panel.setAttribute("id", data.id), panel.feature = this, "tooltiptext" in data ? panel.setAttribute("tooltiptext", data.tooltiptext) : panel.setAttribute("tooltip", "forecastfox-tooltip-" + this._id), "display" in data && (panel.className = data.display), "part" in data && panel.setAttribute("part", data.part), panel;
    }
}), CurrentConditions = Feature.extend({
    _id: "cc",
    data: function(data) {
        if (data) {
            var d = data.cc;
            d.radar = data.radar, d.sunrise_raw = data.forecast[0].day.sun.rise_raw, d.sunset_raw = data.forecast[0].day.sun.set_raw, d.location = data.location, this._data = d;
        }
        return JSON.parse(JSON.stringify(this._data));
    },
    is_night: function() {
        var data = this.data(), to_num = function(a) {
            return a[0] + a[1] / 60;
        }, now = to_num($.map(data.time_raw.split(":"), Number)), rise = to_num($.map(data.sunrise_raw.split(":"), Number)), set = to_num($.map(data.sunset_raw.split(":"), Number));
        return this._logger.debug("now: " + now + " set: " + set + " rise: " + rise), now < rise || now > set;
    },
    general: function(templator, template) {
        this._logger.debug("rendering " + this._id + " general");
        var data = this.data();
        return data["temperature-class"] = "", data["temperature-part"].length == 2 ? data["temperature-class"] += " ff-two" : data["temperature-part"].length == 3 && (data["temperature-class"] += " ff-three"), data["temperature-part"] = data["temperature-part"].join("/"), data.id = this._id, data.forecast = "ff-hidden", data.cc = "", data.links = data.time, data.right = "ff-day-header", data.small_image = this._themes.smallImage(data.icon), data.small_image_default = this._themes.smallImage(data.icon, !0), data.large_image = this._themes.largeImage(data.icon), data.large_image_default = this._themes.largeImage(data.icon, !0), templator.render(template, data);
    },
    details: function(templator, template) {
        this._logger.debug("rendering " + this._id + " details");
        var html = "", prefs = this.preferences(), backing = this.data(), trends = {
            steady: "&mdash;",
            decreasing: "&darr;",
            rising: "&uarr;"
        };
        backing.pressure.trend in trends && (backing.pressure.trend = trends[backing.pressure.trend]);
        var parts = [ [ "features.winds", "${wind-direction} ${wind-speed}", "wind" ], [ "features.humidity", "${humidity}", "humidity" ], [ "features.pressure", "${pressure-value} (<b>${pressure-trend}</b>)", "pressure" ], [ "features.uvindex", "${uv-text} (${uv-index})", "uv" ], [ "features.precipitation", "${precipitation-all}", "precipitation" ], [ "features.visibility", "${visibility}", "visibility" ] ];
        for (var x = 0; x < parts.length; x++) {
            if (!prefs.display[parts[x][2]]) continue;
            var data = {
                label: this._i18n.data(parts[x][0]),
                value: templator.render(parts[x][1], backing)
            };
            html += templator.render(template, data);
        }
        return html;
    },
    toolbar: function(templator, template) {
        this._logger.debug("rendering " + this._id + " toolbar");
        var backing = this.data(), prefs = this.preferences();
        if (!prefs.toolbar.cc || !backing) return "";
        backing.temperature = backing.temperature.join("/"), prefs.toolbar.location_name && (backing.short_day = backing.location.name);
        var data = {
            id: "forecastfox-" + this._id,
            image: this._themes.smallImage(backing.icon),
            defaultimage: this._themes.smallImage(backing.icon, !0),
            link: backing.links.quicklook,
            label: templator.render(this._i18n.data("features.cc.toolbar"), backing)
        };
        return this._create_panel(templator, template, data);
    }
}), Forecast = Feature.extend({
    _day: null,
    _part: null,
    _data: null,
    TIME_DAY: 15,
    init: function(day, part, logging, preferences, i18n, themes) {
        this._id = [ "forecast", day, part ].join("-"), this._super(logging, preferences, i18n, themes), this._day = day, this._part = part, this._validate();
    },
    data: function(data) {
        if (data) if (this._day in data.forecast) this.disabled = !1, this._data = data.forecast[this._day][this._part], this._data.enable || (this._data.enable = {}), this._data.links.hourly == "N/A" && (this._data.enable.hourly = "ff-hidden"); else return this.disabled = !0, {};
        return JSON.parse(JSON.stringify(this._data));
    },
    general: function(templator, template, shouldHide) {
        this._logger.debug("rendering " + this._id + " general");
        var data = this.data();
        return this._part == "night" && shouldHide ? "" : (data["temperature-class"] = this._part == "day" ? "ff-high" : "ff-low", data.temperature.part.length == 2 ? data["temperature-class"] += " ff-two" : data.temperature.part.length == 3 && (data["temperature-class"] += " ff-three"), data.temperature.part = data.temperature.part.join("/"), data.id = this._id, data.forecast = "", data.cc = "ff-hidden", data.small_image = this._themes.smallImage(data.icon), data.small_image_default = this._themes.smallImage(data.icon, !0), data.large_image = this._themes.largeImage(data.icon), data.large_image_default = this._themes.largeImage(data.icon, !0), templator.render(template, data));
    },
    details: function(templator, template) {
        this._logger.debug("rendering " + this._id + " details");
        var parts = {
            day: [ [ "features.maxuv", "${uv-max-text} (${uv-max-index})", "uv" ], [ "features.winds", "${wind-direction} ${wind-speed}", "wind" ], [ "features.precipitation", "${precip-liquid}", "precipitation" ], [ "features.sunrise", "${sun-rise}" ] ],
            night: [ [ "features.winds", "${wind-direction} ${wind-speed}", "wind" ], [ "features.precipitation", "${precip-liquid}", "precipitation" ], [ "features.sunset", "${sun-set}" ], [ "features.moon", "${moon-phase}", "moon" ] ]
        }, html = "", p = parts[this._part], prefs = this.preferences(), backing = this.data();
        for (var x = 0; x < p.length; x++) {
            if (p[x].length > 2 && !prefs.display[p[x][2]]) continue;
            var data = {
                label: this._i18n.data(p[x][0]),
                value: templator.render(p[x][1], backing)
            };
            html += templator.render(template, data);
        }
        return html;
    },
    toolbar: function(templator, template, offset) {
        this._logger.debug("rendering " + this._id + " toolbar");
        var backing = this.data(), prefs = this.preferences(), days_or_nights = prefs.toolbar.days_or_nights, dow = Number(backing.dow) - 1;
        if (!backing) return "";
        var index = this._day * 2 + (this._part == "night" ? 1 : 0), end = prefs.toolbar.days * 2 + offset;
        if (index < offset) return "";
        if (index >= end) return "";
        if (!days_or_nights.match(this._part)) return "";
        backing.temperature.part = backing.temperature.part.join("/");
        var data = {
            id: this._id,
            image: this._themes.smallImage(backing.icon),
            defaultimage: this._themes.smallImage(backing.icon, !0),
            link: backing.links.details,
            label: templator.render(this._i18n.data("features.forecast.toolbar"), backing),
            part: this._part
        };
        return days_or_nights == "days-nights" && this._part == "night" && (data.label = backing.temperature.part), this._create_panel(templator, template, data);
    },
    _validate: function() {
        this._part != "day" && this._part != "night" && this._logger.error("INVALID FORECAST. Valid part: day, night, Current: " + this._part);
    }
}), SWA = Feature.extend({
    _id: "swa",
    _data: null,
    data: function(data) {
        return data && (this._data = data.swa, this._data.location = data.location), this._data;
    },
    toolbar: function(templator, template) {
        var backing = this.data(), prefs = this.preferences();
        if (!prefs.toolbar.swa || !backing || !backing.isActive) return "";
        var data = {
            id: "forecastfox-" + this._id,
            image: this._themes.smallImage("swa", !0),
            display: "image",
            link: backing.link
        };
        return this._create_panel(templator, template, data);
    },
    general: function(templator, template) {
        var data = this.data();
        data.id = this._id, data["class"] = "ff-swa";
        var html = "";
        for (var x = 0; x < data.alerts.length; x++) data.text = data.alerts[x].text, html += templator.render(template, data);
        return html;
    },
    details: function(templator, template) {
        return template;
    },
    message: function(templator, template) {
        this._logger.debug("rendering " + this._id + " message");
        var backing = this.data(), messages = [];
        for (var x = 0; x < backing.alerts.length; x++) {
            var swa = backing.alerts[x];
            messages.push(swa.text);
        }
        if (!backing.isActive) return "";
        var data = {
            "class": "ff-swa",
            link: backing.link,
            text: templator.render(this._i18n.data("features.swa"), {
                message: messages.join(",")
            }),
            type: "swa"
        };
        return templator.render(template, data);
    }
}), Radar = Feature.extend({
    _id: "radar",
    _data: null,
    data: function(data) {
        return data && (this._data = data.cc), this._data;
    },
    toolbar: function(templator, template) {
        var backing = this.data(), prefs = this.preferences(), radar = prefs.toolbar.radar;
        if (radar == "none" || !backing) return "";
        var data = {
            id: "forecastfox-" + this._id,
            image: this._themes.smallImage("radar", !0),
            link: backing.radar.links.regional.animated,
            display: "image"
        };
        return this._create_panel(templator, template, data);
    },
    general: function(templator, template) {
        var data = this.data(), toolbar = this.preferences().toolbar, radar = toolbar.radar;
        data.id = this._id;
        if ([ "small", "medium", "large" ].some(function(o) {
            return o == radar;
        })) data.size = "ff-" + radar, data.src = data.radar.images.regional[radar], data.link = data.radar.links.regional.animated, data.style = ""; else if (radar == "custom") {
            var w = toolbar.radar_custom.width, h = toolbar.radar_custom.height;
            data.size = "", data.src = toolbar.radar_custom.url, data.link = data.radar.links.regional.animated, w != 0 && h != 0 ? data.style = 'width="' + w + 'px" height="' + h + 'px"' : data.style = "";
        }
        return templator.render(template, data);
    },
    details: function(templator, template) {
        return template;
    }
}), Day5Link = Feature.extend({
    _id: "day5",
    data: function(data) {
        return data && (this._data = data.cc), this._data;
    },
    toolbar: function(templator, template) {
        var backing = this.data(), prefs = this.preferences();
        if (!prefs.toolbar.day5 || !backing) return "";
        var data = {
            id: "forecastfox-" + this._id,
            image: this._themes.smallImage("5day", !0),
            link: backing.links.day5,
            tooltiptext: this._i18n.data("features.day5click"),
            display: "image"
        };
        return this._create_panel(templator, template, data);
    }
}), HourlyLink = Feature.extend({
    _id: "hourly",
    data: function(data) {
        return data && (this._data = data.cc), this._data;
    },
    toolbar: function(templator, template) {
        var backing = this.data(), prefs = this.preferences();
        if (!prefs.toolbar.hourly || !backing) return "";
        var data = {
            id: "forecastfox-" + this._id,
            image: this._themes.smallImage("hourly", !0),
            link: backing.links.hourly,
            tooltiptext: this._i18n.data("features.hourlyclick"),
            display: "image"
        };
        return this._create_panel(templator, template, data);
    }
}), ErrorReporter = Feature.extend({
    _id: "error",
    toolbar: function(templator, template, error_active) {
        if (!error_active) return "";
        var data = {
            id: "forecastfox-" + this._id,
            label: this._i18n.data("features.errors.connection"),
            display: "text-image",
            image: "chrome://global/skin/icons/error-16.png",
            tooltiptext: this._i18n.data("features.errors.connection_tooltip")
        };
        return this._create_panel(templator, template, data);
    }
}), ProgressIndicator = Feature.extend({
    _id: "progress",
    toolbar: function(templator, template, location) {
        if (!location) return "";
        var data = {
            id: "forecastfox-" + this._id,
            display: "image",
            image: "chrome://global/skin/icons/loading_16.png",
            tooltiptext: templator.render(this._i18n.data("features.progress"), {
                "location-name": location.name
            })
        };
        return this._create_panel(templator, template, data);
    }
}), DragTarget = Feature.extend({
    _id: "drag-target",
    toolbar: function(templator, template) {
        var data = {
            id: "forecastfox-" + this._id,
            display: "image",
            image: "chrome://forecastfox/skin/images/drag.png",
            tooltiptext: templator.render(this._i18n.data("features.dragdrop.tooltip"))
        };
        return this._create_panel(templator, template, data);
    }
});