var Locations;

(function() {
    var UPDATE_DURATION = {
        forecast: 9e5,
        cc: 9e5
    }, KEY = "locations.", logger, cacheSvc = Components.classes["@mozilla.org/image/cache;1"].createInstance(Components.interfaces.imgICache), ioSvc = Components.classes["@mozilla.org/network/io-service;1"].createInstance(Components.interfaces.nsIIOService);
    function expire_cache(url) {
        var uri = ioSvc.newURI(url, null, null);
        try {
            cacheSvc.removeEntry(uri);
        } catch (e) {}
    }
    Locations = Class.extend({
        init: function(observers, preferences, adapter, processor, datastore) {
            logger = logging.getLogger("locations"), this._preferences = preferences, this._observers = observers, this._adapter = adapter, this._processor = processor, this._datastore = datastore, this._error_count = 1, this._rotation_timer = -1, this._rotation_interval = -1, this._rotation_enabled = -1, this._rotation_paused = !1, this.setup_default_locations();
            var location = this.selected();
            validate(location) && this.update_feed(location);
            var self = this;
            this._observers.add("locale-ready", function() {
                logger.info("Locale has changed.... update feed!"), self.update_feed(self.selected(), !0);
            }), this._observers.add("preference-update", function() {
                var location = self.selected();
                self.update_feed(location);
                var rotate = self._preferences.preference("rotate");
                (rotate.enabled != self._rotation_enabled || rotate.interval != self._rotation_interval) && self.start_rotating();
            }), self.start_rotating(), logger.debug("initialized"), $(function() {
                document.body.addEventListener("online", function() {
                    self.update_feed(self.selected());
                }, !1);
            });
        },
        next: function() {
            var selected = this.selected(), saved = this.saved();
            for (var x = 0; x < saved.length; x++) if (selected.code == saved[x].code) return x == saved.length - 1 ? saved[0] : saved[x + 1];
            return logger.error("locations.next() did not find the next location"), null;
        },
        setup_default_locations: function() {
            if (!this.selected() && this.saved().length == 0) {
                logger.debug("setting the default location to NYC");
                var nyc = {
                    name: "New York, NY",
                    code: "cityId:349727"
                };
                this._storage("saved", [ nyc ]), this._storage("selected", nyc);
            }
            if (!this._preferences.preference("locationmigration")) {
                var self = this, upgrade_closure = function() {
                    self._has_migrate_listener && (document.body.removeEventListener("online", upgrade_closure, !1), self._has_migrate_listener = !1), firefox_migrator.migrate_old_locations(), self._preferences.preference("locationmigration", !0);
                };
                navigator.onLine ? upgrade_closure() : (this._has_migrate_listener = !0, document.body.addEventListener("online", upgrade_closure, !1));
            }
        },
        selected: function(location) {
            var current = this._storage("selected");
            if (location) {
                if (!validate(location) || are_completely_equal(location, current)) return current;
                logger.debug("setting selected location to: " + JSON.stringify("location")), this._storage("selected", location);
                var saved = this.saved(move_to_front(this.saved(), location));
                this.update_feed(location), this._observers.notify("location-changed", {
                    selected: location,
                    saved: saved
                }), this.start_rotating();
            } else location = current;
            return location;
        },
        saved: function(locations) {
            if (locations && $.isArray(locations)) {
                locations = validate_all(locations), locations.sort(function(a, b) {
                    return a.name > b.name;
                }), locations.length == 0 && (locations = [ this.selected() ]), logger.debug("storing saved locations: " + JSON.stringify(locations));
                var old_saved = this._storage("saved");
                this._storage("saved", locations), !contains(locations, this.selected()) && locations.length > 0 && this.selected(locations[0]);
                for (var x = 0, len = old_saved.length; x < len; x++) contains(locations, old_saved[x]) || this._remove_data(old_saved[x].code);
                this._observers.notify("location-changed", {
                    selected: this.selected(),
                    saved: locations
                });
            } else locations = this._storage("saved");
            return locations ? locations : [];
        },
        time_to_expire: function(location, part) {
            var code = location.code, lastUpdate = this.last_update_time(location, part), data = this.original_data(part, location);
            if (!lastUpdate || !data || !("locale" in data) || data.locale != i18n.locale()) lastUpdate = 0;
            var duration = Date.now() - lastUpdate;
            logger.debug("DURATION: " + duration);
            if (duration >= UPDATE_DURATION[part] - 6e4) return logger.debug("location " + code + " " + part + " is expired, duration = 0"), 0;
            if (duration + 6e4 < 0) return logger.debug("location " + code + " " + part + " updated in future, duration = 0"), 0;
            var nextTime = UPDATE_DURATION[part] - duration;
            return logger.debug("feed " + code + " " + part + " duration is " + nextTime / 1e3 / 60 + " min"), nextTime;
        },
        is_error: function() {
            return this._error_count > 1;
        },
        schedule_update: function(location, isError) {
            var self = this, timeToUpdate = Math.min(this.time_to_expire(location, "cc"), this.time_to_expire(location, "forecast"));
            isError ? (timeToUpdate = Math.min(self._error_count, 8) * 60 * 1e3, this._error_count *= 2) : this._error_count = 1, logger.debug("update (" + location.code + ") scheduled for " + timeToUpdate / 6e4 + " min"), clearTimeout(this._timer), this._timer = setTimeout(function() {
                self.update_feed(location);
            }, timeToUpdate + 1e3);
        },
        original_data: function(part, location, data) {
            return location || (location = this.selected()), this._storage(location.code + ".original." + part, data);
        },
        processed_data: function(location, data) {
            location || (location = this.selected());
            if (data) {
                var scratch = $("#scratch_space");
                scratch.length > 0 && (scratch.html(JSON.stringify(data)), data = JSON.parse(scratch.html()), scratch.empty());
            }
            return this._storage(location.code + ".processed", data);
        },
        last_update_time: function(location, part, time) {
            return this._storage([ location.code, part, "time" ].join("."), time);
        },
        _remove_all_data: function() {
            logger.debug("removing all location data");
            var recent = this.saved();
            for (var x = 0, len = recent.length; x < len; x++) this._remove_data(recent[x].code);
        },
        _remove_data: function(code) {
            logger.debug("removing feed data for " + code), this._remove(code + ".original"), this._remove(code + ".processed"), this._remove(code + ".cc.time"), this._remove(code + ".forecast.time"), this._remove(code + ".time");
        },
        start_rotating: function() {
            logger.debug("Starting to rotate profiles");
            var rotate = this._preferences.preference("rotate"), interval = Math.max(.25, rotate.interval), self = this;
            this._rotation_interval = rotate.interval, this._rotation_enabled = rotate.enabled;
            if (!rotate.enabled || isNaN(interval) || interval <= 0) {
                clearTimeout(this._rotation_timer), logger.debug("[rotate] not rotating locations");
                return;
            }
            logger.debug("[rotate] starting to rotate locations every " + interval + " minutes"), clearTimeout(this._rotation_timer), this._rotation_timer = setTimeout(function() {
                try {
                    self._rotation_paused || self.selected(self.next());
                } catch (e) {
                    logger.error(e);
                }
            }, interval * 6e4);
        },
        pause_rotating: function() {
            logger.debug("paused rotating");
            var self = this;
            this._rotation_paused = !0, this._rotation_paused_timer = setTimeout(function() {
                logger.debug("rotation pause timed out. resuming"), self.resume_rotating();
            }, 3e5);
        },
        resume_rotating: function() {
            logger.debug("resumed rotating"), this._rotation_paused = !1, clearTimeout(this._rotation_paused_timer), this.start_rotating();
        },
        _remove: function(key) {
            this._datastore.remove(KEY + key);
        },
        _storage: function(key, data) {
            return data !== undefined && this._datastore.set(KEY + key, data, !0), this._datastore.get(KEY + key, !0);
        },
        update_feed: function(location, force) {
            logger.debug("updating feed for " + location.code);
            var code = location.code, name = location.name, self = this;
            this._timer && (clearTimeout(this._timer), this._timer = null);
            if (this._updates_disabled) {
                logger.debug("updates disabled");
                return;
            }
            $.when(this.update_part(location, "cc", force), this.update_part(location, "forecast", force)).done(function() {
                logger.debug("locations main callback");
                var data = {
                    cc: self.original_data("cc", location),
                    forecast: self.original_data("forecast", location)
                };
                try {
                    var processed = self._processor.process_data(data, location);
                    self.processed_data(location, processed), self.update_caches(processed), self._observers.notify("feed-ready", code), self.schedule_update(location, !1);
                } catch (e) {
                    logger.error("data processing failed. invalid. removing data", e), logger.error(e), self.last_update_time(location, "cc", 1), self.last_update_time(location, "forecast", 1), self.schedule_update(location, !0);
                }
            }).fail(function(error_code) {
                logger.error("locations update failed"), self._observers.notify("feed-error", error_code), self.schedule_update(location, !0);
            });
        },
        update_part: function(location, part, force) {
            var ttl = this.time_to_expire(location, part), self = this;
            return !force && ttl > 0 && this.original_data(part, location) ? $.Deferred().resolve() : (logger.debug("updating " + part), this._observers.notify("feed-updating", location), this._adapter.fetch_feed_data(location.code, part).done(function(data) {
                logger.debug(part + " data received for " + location.code), data.locale = i18n.locale(), self.original_data(part, location, data), self.last_update_time(location, part, Date.now());
            }));
        },
        update_caches: function(data) {
            expire_cache(data.radar.images.regional.large), expire_cache(data.radar.images.regional.medium), expire_cache(data.radar.images.regional.small), expire_cache(this._preferences.preference("toolbar").radar_custom.url);
        }
    });
    function are_completely_equal(loc1, loc2) {
        return loc1.code == loc2.code && loc1.name == loc2.name;
    }
    function move_to_front(list, location) {
        return list.some(function(l) {
            return l.code == location.code;
        }) ? list.some(function(l) {
            return are_completely_equal(l, location);
        }) ? list : [ location ].concat(list.filter(function(l) {
            return l.code != location.code;
        })) : [ location ].concat(list);
    }
    function unique(list) {
        var new_list = [];
        return list.forEach(function(loc) {
            if (contains(new_list, loc)) return;
            new_list.push(loc);
        }), new_list;
    }
    function contains(list, location) {
        return list.some(function(l) {
            return l.code == location.code;
        });
    }
    function validate_all(list) {
        return unique(list.filter(function(l) {
            return validate(l);
        }));
    }
    function validate(location) {
        return !!location && typeof location == "object" && "code" in location ? !0 : (logger.error("location failed validation: " + JSON.stringify(location)), !1);
    }
})();