/*******************************************************************************
 * Available logging levels.
 *
 * @version 1.0
 ******************************************************************************/const LoggingLevels = {
    NOTSET: 0,
    DEBUG: 10,
    INFO: 20,
    WARN: 30,
    ERROR: 40,
    CRITICAL: 50
};

var Logging = Class.extend({
    loggers: null,
    configs: null,
    level: LoggingLevels.NOTSET,
    init: function(level, configs) {
        this.loggers = {}, this.configure(level, configs), this._logger = this.getLogger("root"), this._logger.debug("initialized");
    },
    configure: function(level, configs) {
        typeof level == "number" && (this.level = level);
        if (!configs) return;
        $.isArray(configs) ? this.configs = configs : this.configs = [ configs ];
    },
    getLogger: function(name) {
        name in this.loggers || (this.loggers[name] = []);
        var logger = new Logger(name);
        logger.setLevel(this.level);
        var handlers = [];
        if (!this.configs || this.configs.length == 0) {
            var handler = new LogHandler, formatter = new LogFormatter;
            handler.setFormatter(formatter), handlers.push(handler);
        } else this.configs.forEach(function(config) {
            var handler, formatter, level;
            "formatter" in config ? formatter = new config.formatter : formatter = new LogFormatter, "format" in config && formatter.setFormat(config.format), "level" in config ? level = config.level : level = LoggingLevels.NOTSET, "handler" in config ? handler = new config.handler(level, formatter) : handler = new LogHandler(level, formatter), handlers.push(handler);
        });
        return logger.handlers = handlers, this.loggers[name].push(logger), logger;
    },
    removeLogger: function(logger) {
        if (!!logger && logger.name in this.loggers) this.loggers[logger.name] = this.loggers[logger.name].filter(function(l) {
            return l == logger ? (logger.handlers.forEach(function(handler) {
                handler.flush(), handler.close();
            }), !1) : !0;
        }); else return;
    },
    debug: function(message) {
        this.log(LoggingLevels.DEBUG, message);
    },
    info: function(message) {
        this.log(LoggingLevels.INFO, message);
    },
    warn: function(message) {
        this.log(LoggingLevels.WARN, message);
    },
    error: function(message) {
        this.log(LoggingLevels.ERROR, message);
    },
    critical: function(message) {
        this.log(LoggingLevels.CRITICAL, message);
    },
    exception: function(message, e) {
        this.log(LoggingLevels.ERROR, message, e);
    },
    log: function(level, message, exception) {
        this._logger.log(level, message, exception);
    },
    shutdown: function() {
        this._logger.debug("logging has shut down");
        for (var name in this.loggers) {
            var loggerList = this.loggers[name];
            for (var x = 0; x < loggerList.length; x++) loggerList[x].handlers.forEach(function(handler) {
                handler.flush(), handler.close();
            });
        }
        this.configs = null, this._logger = null, this.loggers = {};
    }
}), Logger = Class.extend({
    name: "",
    level: LoggingLevels.NOTSET,
    handlers: null,
    init: function(name, level) {
        this.handlers = [], this.name = name, this.setLevel(level);
    },
    setLevel: function(level) {
        typeof level == "number" && (this.level = level);
    },
    isEnabledFor: function(level) {
        return level >= this.level ? !0 : !1;
    },
    debug: function(message) {
        this.log(LoggingLevels.DEBUG, message);
    },
    info: function(message) {
        this.log(LoggingLevels.INFO, message);
    },
    warn: function(message) {
        this.log(LoggingLevels.WARN, message);
    },
    error: function(message) {
        this.log(LoggingLevels.ERROR, message);
    },
    critical: function(message) {
        this.log(LoggingLevels.CRITICAL, message);
    },
    exception: function(message, e) {
        this.log(LoggingLevels.ERROR, message, e);
    },
    log: function(level, message, exception) {
        if (this.isEnabledFor(level)) {
            var record = new LogRecord(this.name, level, message, exception);
            if (this.handlers.length == 0) {
                (new LogHandler).handle(record);
                return;
            }
            this.handlers.forEach(function(handler) {
                handler.handle(record);
            });
        }
    },
    addHandler: function(handler) {
        handler instanceof LogHandler && (this.handlers.some(function(o) {
            return o === handler;
        }) || this.handlers.push(handler));
    },
    removeHandler: function(handler) {
        this.handlers = this.handlers.filter(function(o) {
            return o != handler;
        });
    }
}), LogHandler = Class.extend({
    level: 0,
    formatter: null,
    init: function(level, formatter) {
        this.setLevel(level), formatter || (formatter = new LogFormatter), this.setFormatter(formatter);
    },
    setLevel: function(level) {
        typeof level == "number" && (this.level = level);
    },
    setFormatter: function(formatter) {
        formatter instanceof LogFormatter && (this.formatter = formatter);
    },
    flush: function() {},
    close: function() {},
    handle: function(record) {
        record.level >= this.level && this.emit(record);
    },
    format: function(record, exclude) {
        var formatted = this.formatter.format(record);
        return exclude !== !0 && record.exception && (formatted += "\n" + this.formatter.formatException(record.exception)), formatted;
    },
    emit: function(record) {}
}), LogFormatter = Class.extend({
    frmt: "%(message)",
    levelNames: {},
    init: function(frmt) {
        this.setFormat(frmt);
        for (var name in LoggingLevels) this.levelNames[LoggingLevels[name]] = name;
    },
    setFormat: function(frmt) {
        typeof frmt == "string" && (this.frmt = frmt);
    },
    format: function(record) {
        var formatted = this.frmt;
        return formatted = formatted.replace(/%\(name\)/g, record.name), formatted = formatted.replace(/%\(levelnumber\)/g, record.level), formatted = formatted.replace(/%\(level\)/g, this.getLevelName(record.level)), formatted = formatted.replace(/%\(when\)/g, record.when.toUTCString()), formatted = formatted.replace(/%\(message\)/g, record.message), formatted;
    },
    formatException: function(exception) {
        var formatted = "";
        for (var attr in exception) formatted += attr + ": " + exception[attr] + "\n";
        return formatted;
    },
    getLevelName: function(level) {
        return this.levelNames[level] ? this.levelNames[level] : "";
    }
}), LogRecord = Class.extend({
    name: "",
    level: 0,
    message: "",
    exception: null,
    when: null,
    init: function(name, level, message, exception) {
        this.name = name, this.level = level, this.message = message, this.exception = exception, this.when = new Date;
    }
});