/*******************************************************************************
 * Browser specific log handlers.  This module currently implements:
 *
 * AlertLogHandler
 * ConsoleLogHandler
 * StorageLogHandler
 *
 * NOTE:  The storage log handler deals with the sqlite directly since
 *        datastore depends on logging.
 *
 * @see /content/logging.js
 * @see /content/sqlite.js
 * @see /content/external/class.js
 * @version 1.0
 ******************************************************************************/var AlertLogHandler = LogHandler.extend({
    _mediator: null,
    init: function(level, formatter) {
        this._super(level, formatter), this._mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    },
    emit: function(record) {
        var win = this._mediator.getMostRecentWindow(null);
        if (!win) return;
        win.alert(this.format(record));
    },
    close: function() {
        this._mediator = null;
    }
}), ConsoleLogHandler = LogHandler.extend({
    _console: null,
    init: function(level, formatter) {
        this._super(level, formatter), this._console = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
    },
    emit: function(record) {
        record.level >= LoggingLevels.ERROR ? record.exception ? (this._console.logStringMessage(this.format(record, !0)), record.expection instanceof Components.interfaces.nsIConsoleMessage || (record.exception = this._createError(record.exception)), this._console.logMessage(record.exception)) : this._console.logStringMessage(this.format(record)) : this._console.logStringMessage(this.format(record));
    },
    close: function() {
        this._console = null;
    },
    _createError: function(message) {
        var stack = this._findStack(), se = Components.classes["@mozilla.org/scripterror;1"].createInstance(Components.interfaces.nsIScriptError);
        return se.init(message, stack.filename, stack.sourceLine, stack.lineNumber, 0, Components.interfaces.nsIScriptError.errorFlag, "javascript"), se;
    },
    _findStack: function() {
        var rv = {
            filename: null,
            sourceLine: null,
            lineNumber: 0
        }, stack = Components.stack;
        while (stack) {
            if (stack.filename.indexOf("loghandlers.js") != -1) {
                stack = stack.caller;
                continue;
            }
            if (stack.filename.indexOf("logging.js") != -1) {
                stack = stack.caller;
                continue;
            }
            break;
        }
        return stack && (rv.filename = stack.filename, rv.sourceLine = stack.sourceLine, rv.lineNumber = stack.lineNumber), rv;
    }
}), StorageLogHandler = LogHandler.extend({
    _key: "logging",
    _storage: null,
    _limit: 500,
    init: function(level, formatter) {
        this._super(level, formatter);
        var scope = {}, loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
        loader.loadSubScript("chrome://forecastfox/content/sqlite.js", scope), this._storage = new scope.SQLite;
    },
    format: function(record) {
        return {
            when: record.when.toUTCString(),
            name: record.name,
            number: record.level,
            level: this.formatter.getLevelName(record.level),
            message: record.message,
            formatted: this._super(record)
        };
    },
    emit: function(record) {
        var results = this._storage.query("get", "SELECT value FROM datastore WHERE key=:key", {
            key: this._key
        }), records = null;
        results.length > 0 && (records = JSON.parse(results[0].value)), records || (records = []), records.unshift(this.format(record)), records = records.slice(0, this._limit);
        var value = JSON.stringify(records);
        results.length > 0 ? this._storage.execute("update", "UPDATE datastore SET value=:value WHERE key=:key;", {
            key: this._key,
            value: value
        }) : this._storage.execute("insert", "INSERT INTO datastore (key, value) VALUES(:key, :value);", {
            key: this._key,
            value: value
        });
    },
    close: function() {
        this._storage = null;
    }
});