/*******************************************************************************
 * Observer service is a way for services to communicate with each other
 * without having to actually reference each other.  A service can register
 * to listen for a topic, with a callback.  When the topic is fired a json
 * object is passed into the callback function.  Calling the notify function
 * on the service will loop through the observers for a topic and call the
 * callback functions that have been registered.
 *
 * @see /content/external/jquery-1.3.2.js
 * @see /content/external/class.js
 * @version 1.0
 ******************************************************************************/const OBSERVERS_CHANNEL = "observers";

var Observers = Class.extend({
    _prefix: "forecastfox-",
    init: function(logging, channel, background) {
        this._logger = logging.getLogger("observers"), this._callbacks = {}, this._ports = [], channel || (channel = OBSERVERS_CHANNEL), this._channel = channel, this._installListeners(), this._logger.debug("adding observer " + this._topic()), this._logger.debug("initialized");
    },
    add: function(topic, callback) {
        topic in this._callbacks || (this._callbacks[topic] = [], this._logger.debug("topic storage created for " + topic));
        if (this._callbacks[topic].some(function(c) {
            return c == callback;
        })) {
            this._logger.debug("callback already stored for " + topic);
            return;
        }
        this._callbacks[topic].push(callback), this._logger.debug("callback stored for " + topic);
    },
    remove: function(topic, callback) {
        if (topic in this._callbacks) this._callbacks[topic] = this._callbacks[topic].filter(function(o) {
            return o != callback;
        }), this._logger.debug("callback remove from " + topic); else {
            this._logger.debug("no callbacks stored for " + topic);
            return;
        }
    },
    notify: function(topic, data) {
        this._send({
            topic: topic,
            data: data
        });
    },
    shutdown: function() {
        this._service.removeObserver(this, this._topic(), !0), this._callbacks = {}, this._logger.debug(this._channel + " observers has shut down");
    },
    _notify: function(topic, data) {
        var self = this, callbacks = this._callbacks[topic];
        if (!callbacks) {
            this._logger.debug("no callbacks stored for " + topic);
            return;
        }
        callbacks.forEach(function(callback) {
            try {
                setTimeout(function() {
                    callback.apply(null, [ data ]);
                }, 0);
            } catch (e) {
                self._logger.exception("callback failed for " + topic, e);
            }
        });
    },
    _send: function(msg) {
        var data = JSON.stringify(msg);
        this._logger.debug("sending message: " + msg.topic), this._service.notifyObservers(this, this._topic(), data);
    },
    _installListeners: function() {
        this._service = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService), this._service.addObserver(this, this._topic(), !1);
    },
    _topic: function() {
        return this._prefix + this._channel;
    },
    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsISupports) || iid.equals(Components.interfaces.nsISupportsWeakReference) || iid.equals(Components.interfaces.nsIObserver)) return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },
    observe: function(subject, topic, data) {
        this._logger.debug("received message: " + topic);
        var msg = JSON.parse(data);
        this._notify(msg.topic, msg.data);
    }
});