/*******************************************************************************
 * The preference object manages Forecastfox user preferences. Preferences are
 * defined Firefox style, as key-value pairs where the key is a "." delimited
 * heirarchy of names.
 *
 * Clients may listen for updated preferences on the "preference-update" topic
 * via the observer service (PREFERENCES_UPDATE_TOPIC event). For this event,
 * the data is a map of key-value pairs for all the changed preferences.
 *
 * @see /content/logging.js
 * @see /content/observers.js
 * @see /content/datastore.js
 * @see /content/external/jquery-1.3.2.js
 * @see /content/external/class.js
 * @version 1.0
 ******************************************************************************/const PREFERENCES_UPDATE_TOPIC = "preference-update";

var RADAR_EXAMPLE_URL = "http://sirocco.accuweather.com/nx_mosaic_400x300_public/sir/inmasirCT_.gif";

const PREFERENCES_DEFAULTS = {
    version: "17",
    defaults: !0,
    showoptions: !0,
    firefoxmigration: !1,
    locationmigration: !1,
    show_english_locations: !1,
    unitsystem: "american",
    spring_inserted: !1,
    rotate: {
        enabled: !1,
        interval: 1
    },
    theme: {
        icon_pack: 1,
        separators: $.client.os != "Mac"
    },
    units: {
        temperature: [ "f" ],
        speed: [ "mph" ],
        time: [ "h12" ],
        pressure: [ "inhg" ],
        distance: {
            shrt: [ "inches" ],
            lng: [ "mi" ]
        }
    },
    display: {
        humidity: !0,
        precipitation: !1,
        pressure: !0,
        uv: !1,
        visibility: !1,
        wind: !0,
        quality: !1,
        moon: !1
    },
    toolbar: {
        days: 2,
        location_name: !1,
        days_or_nights: "days",
        cc: !0,
        radar: "medium",
        radar_custom: {
            url: RADAR_EXAMPLE_URL,
            width: null,
            height: null
        },
        day5: !0,
        hourly: !0,
        swa: !0,
        parent: "DEFAULT",
        position: "last",
        display: 9,
        force_addonbar_visible: 0
    }
}, PREFERENCES_UPGRADE = {
    _count: 20,
    "1.5.0": function(service) {
        return "1.5.1";
    },
    "1.5.2": function(service) {
        return this["1.5.1"](service);
    },
    "1.5.1": function(service) {
        var selected = service.preference("location"), saved = service._datastore.get("locations.saved", !0), defaults = PREFERENCES_DEFAULTS.locations, locations = service.preference("locations");
        if (!locations || !("selected" in locations) || !("saved" in locations)) {
            var obj = {};
            selected ? obj.selected = selected : "selected" in locations ? obj.selected = locations.selected : obj.selected = defaults.selected, saved && $.isArray(saved) ? obj.saved = saved : "saved" in locations && $.isArray(locations.saved) ? obj.saved = locations.saved : obj.saved = defaults.saved, service.preference("locations", obj);
        }
        service.remove("location"), service._datastore.remove("locations.saved");
        var units = service.preference("units");
        return "long" in units.distance && (units.distance.lng = units.distance["long"], delete units.distance["long"]), "short" in units.distance && (units.distance.shrt = units.distance["short"], delete units.distance["short"]), service.preference("units", units), service.preference("showoptions", !1), "2";
    },
    "2.0": function(service) {
        return this["1.5.1"](service);
    },
    "2": function(service) {
        var defaults = PREFERENCES_DEFAULTS;
        service.preference("show_english_locations", defaults.show_english_locations), service.preference("showoptions", !0), service.preference("toolbar", defaults.toolbar), service.preference("theme", defaults.theme), service.preference("days") && service.remove("days");
        var locations = service.preference("locations");
        return locations && "selected" in locations && service._datastore.set("locations.selected", locations.selected, !0), locations && "saved" in locations && service._datastore.set("locations.saved", locations.saved, !0), service.remove("locations"), "3";
    },
    "3": function(service) {
        var ds = service._datastore, saved = ds.get("locations.saved", !0);
        for (var x = 0; x < saved.length; x++) {
            var prefix = "locations." + saved[x].code;
            ds.remove(prefix + ".original"), ds.remove(prefix + ".processed"), ds.remove(prefix + ".cc.time"), ds.remove(prefix + ".forecast.time"), ds.remove(prefix + ".time");
        }
        ds.remove("locations.saved"), ds.remove("locations.selected");
        var toolbar = service.preference("toolbar");
        return toolbar.radar = !0, service.preference("toolbar", toolbar), service.preference("showoptions", !0), "4";
    },
    "4": function(service) {
        var toolbar = service.preference("toolbar");
        return toolbar.display = toolbar.display == "text-image" ? 5 : 0, toolbar.separators = !0, service.preference("toolbar", toolbar), "5";
    },
    "5": function(service) {
        var toolbar = service.preference("toolbar");
        return toolbar.parent = toolbar.position, toolbar.position = "last", service.preference("toolbar", toolbar), service.preference("locationmigration", !1), "6";
    },
    "6": function(service) {
        return service.preference("installed", (new Date).valueOf()), "7";
    },
    "7": function(service) {
        var toolbar = service.preference("toolbar");
        return toolbar.radar = toolbar.radar ? "large" : "none", service.preference("toolbar", toolbar), "8";
    },
    "8": function(service) {
        var theme = service.preference("theme"), toolbar = service.preference("toolbar");
        return theme.icon_pack = 1, theme.separators = toolbar.separators, delete theme.icon_pack_id, delete toolbar.separators, service.preference("theme", theme), service.preference("toolbar", toolbar), "9";
    },
    "9": function(service) {
        return "10";
    },
    "10": function(service) {
        var installed = service.preference("installed");
        return service._datastore.set("analytics-installed", installed + ""), service.remove("installed"), "11";
    },
    "11": function(service) {
        var rotate = PREFERENCES_DEFAULTS.rotate, v, toolbar = service.preference("toolbar");
        (v = firefox_migrator.get_old_pref("profile.switch.enabled", "Bool")) !== null && (rotate.enabled = v), (v = firefox_migrator.get_old_pref("profile.switch.delay", "Int")) !== null && (rotate.interval = v), rotate.enabled ? toolbar.location_name = !0 : toolbar.location_name = !1, toolbar.days_or_nights = "days";
        var mode = [ "days", "nights", "days-nights" ];
        return (v = firefox_migrator.get_old_pref("dayf.panel.mode", "Int")) !== null && v >= 0 && v <= 2 && (toolbar.days_or_nights = mode[v]), toolbar.display == 5 && (toolbar.display = 9), service.preference("rotate", rotate), service.preference("toolbar", toolbar), "12";
    },
    "12": function(service) {
        var ds = service._datastore, saved = ds.get("locations.saved", !0);
        for (var x = 0, len = saved.length; x < len; x++) {
            var prefix = "locations." + saved[x].code;
            ds.remove(prefix + ".original"), ds.remove(prefix + ".processed"), ds.remove(prefix + ".cc.time"), ds.remove(prefix + ".forecast.time");
        }
        return saved.sort(function(a, b) {
            return a.name > b.name;
        }), ds.set("locations.saved", saved, !0), service.preference("spring_inserted", !1), "13";
    },
    "13": function(service) {
        var toolbar = service.preference("toolbar");
        return toolbar.radar == "small" && (toolbar.radar = "medium"), service.preference("toolbar", toolbar), "14";
    },
    "14": function(service) {
        return "15";
    },
    "15": function(service) {
        var toolbar = service.preference("toolbar");
        return toolbar.radar == "small" && (toolbar.radar = "medium"), toolbar.radar_custom = PREFERENCES_DEFAULTS.toolbar.radar_custom, service.preference("toolbar", toolbar), "16";
    },
    "16": function(service) {
        return service.preference("force_addonbar_visible", 0), "17";
    }
};

var Preferences = Class.extend({
    _key: "preferences",
    _logger: null,
    _observers: null,
    _datastore: null,
    _cache: null,
    init: function(logging, observers, datastore, preferences) {
        this._logger = logging.getLogger("preferences"), this._observers = observers, this._datastore = datastore, this._migrating = !1, preferences && this.preferences(preferences), this.migrate(), this._cache = this._datastore.get(this._key, !0), this._logger.debug("initialized");
    },
    migrate: function() {
        this._logger.debug("migrating preferences"), this._migrating = !0;
        var should_notify = !1;
        try {
            var data = this._getAll();
            this._validate(data) || (should_notify = !0, this._logger.debug("setting the default preferences"), this.preferences(PREFERENCES_DEFAULTS), "firefox_migrator" in window && firefox_migrator.migrate_old_preferences(this));
            var version = this.preference("version"), count = 0;
            while (version != PREFERENCES_DEFAULTS.version) {
                should_notify = !0, count++, this._logger.debug("upgrading preferences from version " + version + " ...");
                if (count > PREFERENCES_UPGRADE._count) {
                    this._logger.error("unexpected result during migration. reverting to defaults"), this.preferences(PREFERENCES_DEFAULTS);
                    break;
                }
                version in PREFERENCES_UPGRADE ? version = PREFERENCES_UPGRADE[version](this) : version = PREFERENCES_DEFAULTS.version, this._logger.debug("upgrade to version " + version + " complete"), this.preference("version", version);
            }
            this.preference("showoptions") && Forecastfox.customize();
        } catch (e) {
            this._logger.exception("Migration error", e), this.preferences(PREFERENCES_DEFAULTS);
        } finally {
            this._migrating = !1, should_notify && this._observers.notify(PREFERENCES_UPDATE_TOPIC, this.preferences());
        }
    },
    preference: function(key, value) {
        var data = {};
        return value !== undefined && (data[key] = value, this._setAll(data)), data = this._getAll(), key in data ? data[key] : null;
    },
    preferences: function(preferences) {
        return preferences && this._setAll(preferences), this._getAll();
    },
    remove: function(key) {
        if (typeof key != "string") return null;
        var data = this._getAll();
        if (key in data) {
            var value = data[key];
            return delete data[key], this._cache = data, this._datastore.set(this._key, data, !0), this._migrating || (this._observers.notify(PREFERENCES_UPDATE_TOPIC, {
                key: value
            }), this._logger.debug("removed preference, notifying observers: " + key)), value;
        }
        return this._logger.debug("removing preference that doesn't exist: " + key), null;
    },
    _getAll: function() {
        var data = this._datastore.get(this._key, !0);
        return !this._validate(data) && this._validate(this._cache) && (data = this._cache, this._datastore.set(this._key, data, !0)), data ? data : {};
    },
    _setAll: function(preferences) {
        if (!this._validate(preferences)) return !1;
        var data = this._getAll();
        for (var key in preferences) data[key] = preferences[key];
        return this._cache = data, this._datastore.set(this._key, data, !0), this._migrating || (this._logger.debug("preferences updated, notifying observers: " + JSON.stringify(preferences)), this._observers.notify(PREFERENCES_UPDATE_TOPIC, preferences)), !0;
    },
    _validate: function(data) {
        if (typeof data != "object") return this._logger.warn("data not an object"), !1;
        var count = 0;
        for (var key in data) {
            count++;
            break;
        }
        return count === 0 ? (this._logger.warn("no keys set in data"), !1) : !0;
    }
});