/*******************************************************************************
 * Object used to interact with Firefox's sqlite implementation.  This object
 * caches storage statements to optimize reuse.  You should call either query or
 * execute passing in name, sql, and params.
 *
 * @see /content/logging.js
 * @see /content/external/class.js
 * @version 1.0
 ******************************************************************************/var SQLite = Class.extend({
    _logger: null,
    _statements: null,
    _storage: null,
    database: function() {
        var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
        return file.append("forecastfox.sqlite"), file;
    },
    init: function(logging) {
        logging && (this._logger = logging.getLogger("sqlite"));
        var observers = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        observers.addObserver(this, "xpcom-shutdown", !0), this._logger && this._logger.debug("opening connection to forecastfox.sqlite");
        var ss = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);
        this._storage = ss.openDatabase(this.database()), this._storage.tableExists("datastore") || (this._logger && this._logger.debug("creating datastore table"), this._storage.createTable("datastore", "key TEXT PRIMARY KEY NOT NULL, value TEXT")), this._logger && this._logger.debug("initialized");
    },
    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsISupports) || iid.equals(Components.interfaces.nsISupportsWeakReference) || iid.equals(Components.interfaces.nsIObserver)) return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },
    observe: function(subject, topic, data) {
        if (topic != "xpcom-shutdown") return;
        this._storage.close(), this._storage = null;
    },
    statement: function(name, sql) {
        return this._storage.createStatement(sql);
    },
    execute: function(name, sql, params) {
        var statement = null;
        try {
            statement = this.statement(name, sql);
        } catch (e) {
            return this._logger && this._logger.exception("error creating statement", e), !1;
        }
        if (params !== undefined) for (var key in params) statement.params[key] = params[key];
        try {
            this._logger && this._logger.debug("executing " + sql), statement.execute();
        } catch (e) {
            return this._logger && this._logger.exception("error executing", e), !1;
        }
        return !0;
    },
    query: function(name, sql, params) {
        var statement = null;
        try {
            statement = this.statement(name, sql);
        } catch (e) {
            return this._logger && this._logger.exception("error creating statement", e), [];
        }
        if (params !== undefined) for (var key in params) statement.params[key] = params[key];
        var results = null;
        try {
            results = this._results(statement);
        } catch (e) {
            return this._logger && this._logger.exception("error querying", e), [];
        } finally {
            statement.reset();
        }
        return results;
    },
    _results: function(statement) {
        var results = [], columns = null;
        while (statement.step()) {
            if (!columns) {
                columns = [];
                for (var j = 0; j < statement.columnCount; j++) columns[j] = statement.getColumnName(j);
            }
            var result = {};
            for (var i = 0; i < columns.length; i++) result[columns[i]] = statement.row[columns[i]];
            results[results.length] = result;
        }
        return results;
    }
});